{% extends '_base.html' %}

{% from 'macros/form.html' import render_field %}
{% from 'macros/form.html' import render_button %}

{% block content %}{{ super() }}
<h1>{{ _('Login') }}</h1>
<form action="{{ url_for_security('login') }}" method="post" name="login_user_form">
  {{ login_user_form.hidden_tag() }}

  {{ render_field(login_user_form.email, _('Enter your Ascoderu email address or account name')) }}
  {{ render_field(login_user_form.password, _('Enter your password')) }}
  {{ render_button(login_user_form.submit, _('Login')) }}
</form>

{{ _("Don't have an account?") }}
<a href="{{ url_for_security('register') }}">{{ _('Register now!') }}</a>
{% endblock %}
